"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.canonize = exports.getCanon = exports.canonicalize = exports.getAliasFrom = exports.pureAliasedListeners = exports.alias = undefined;

var _fUtility = require("f-utility");

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
@method alias
@param {Object} object - an object to store aliases in
@param {string} original - a string
@param {string} alt - the alias
@returns null
*/
var alias = exports.alias = (0, _fUtility.curry)(function (object, original, alt) {
  /* istanbul ignore next */
  if (!object[alt]) {
    /* istanbul ignore next */
    object[alt] = original; // eslint-disable-line fp/no-mutation
  }
  /* istanbul ignore next */
  if (!object[original]) {
    /* istanbul ignore next */
    object[original] = object[alt]; // eslint-disable-line fp/no-mutation
  }
});

/**
 @method pureAliasedListeners
 @param {Function} subscriber - a callback function
 @returns {Object} current
 */
var pureAliasedListeners = exports.pureAliasedListeners = function pureAliasedListeners(subscriber) {
  return (0, _fUtility.curry)(function (original, alt, seed) {
    var _merge;

    var emitted = (0, _fUtility.merge)(seed, (_merge = {}, _defineProperty(_merge, alt, original), _defineProperty(_merge, original, original), _merge));
    subscriber(emitted);
    return emitted;
  });
};

/**
@method getAliasFrom
@param {Object} object - an object to pull aliases from
@param {string} key - a key to look up
@returns {*} whatever the lookup resulted in or the key itself
*/
var getAliasFrom = exports.getAliasFrom = (0, _fUtility.curry)(function (object, key) {
  return object && object[key] || key;
});

/**
@method canonicalize
@param {Object} object - an object to store aliases in and pull aliases from
@returns {Object} an object with canonize and getCanon on it
*/
var canonicalize = exports.canonicalize = function canonicalize(object) {
  return {
    canonize: function canonize(a) {
      var b = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : a;
      return alias(object, a, b);
    },
    getCanon: getAliasFrom(object)
  };
};

var authors = {};

var _canonicalize = canonicalize(authors);

var getCanon = _canonicalize.getCanon,
    canonize = _canonicalize.canonize;
exports.getCanon = getCanon;
exports.canonize = canonize;