"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.filterByStringPattern = exports.stringMatcher = exports.isAMergeCommit = exports.anyFilesMatchFromObject = exports.matchesWildcards = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fUtility = require("f-utility");

var _micromatch = require("micromatch");

var _micromatch2 = _interopRequireDefault(_micromatch);

var _utils = require("./utils");

var _alias = require("./alias");

var _constants = require("./constants");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var ASTERISK = _constants.CHARACTER_LITERALS.ASTERISK,
    TILDE = _constants.CHARACTER_LITERALS.TILDE;

/**
@method matchesWildcards
@param {Array} wildcards - a list of wildcards to match against
@param {Array} list - a list of files to compare against
@return {boolean} whether any of the files match any of the wildcards
*/

var matchesWildcards = exports.matchesWildcards = (0, _fUtility.curry)(function (wildcards, list) {
  return _micromatch2.default.some(list, (0, _utils.neue)(wildcards));
});

/**
@method anyFilesMatchFromObject
@param {Object} changes - an object whose keys are lists of files
@param {Array} wildcards - a list of wildcards
@return {boolean} whether any keys on the changes object match any of the wildcards
*/
var anyFilesMatchFromObject = exports.anyFilesMatchFromObject = (0, _fUtility.curry)(function (changes, wildcards) {
  return (0, _fUtility.pipe)(_fUtility.entries, (0, _fUtility.reduce)(function (agg, _ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        v = _ref2[1];

    return agg || matchesWildcards(wildcards, v);
  }, false))(changes);
});
var MERGE_WORD = "Merge ";
/**
@method isAMergeCommit
@param {Object} x - an object with an optional subject
@returns {boolean} whether the given object's subject starts with 'Merge '
*/
var isAMergeCommit = exports.isAMergeCommit = function isAMergeCommit(x) {
  return (0, _fUtility.pathOr)("", ["subject"], x).substr(0, 6) === MERGE_WORD;
};

var stringMatcher = exports.stringMatcher = (0, _fUtility.curry)(function (commit, _ref3) {
  var _ref4 = _slicedToArray(_ref3, 2),
      k = _ref4[0],
      v = _ref4[1];

  if (v === "true" || v === "false") {
    v = !!v; // eslint-disable-line fp/no-mutation
  }
  var dotted = (0, _utils.indexAny)(".", k);
  if (dotted || commit && commit[k]) {
    var value = dotted ? (0, _fUtility.path)(k.split("."), commit) : commit[k];
    // authors are magical
    if (k === "author" || k === "authorName") {
      return (0, _alias.getCanon)(value) === (0, _alias.getCanon)(v);
    }
    // asterisks turn on minimatch mode for arrays
    if ((0, _fUtility.isArray)(value) && (0, _utils.indexAny)(ASTERISK, v)) {
      return _micromatch2.default.some(value, v);
    }
    // a ~ suffix will do looser matching
    if (TILDE.test(v)) {
      return (0, _utils.indexAny)(v.replace(TILDE, ""), value.toLowerCase());
    }
    return value === v;
  }
});
/**
 @method filterByStringPattern
 @param {string} f - colon and hash character delimited string (e.g. 'a:1#b:2')
 @param {Array} commits - an array of commits
 @return {Array} a potentially filtered array of commits
 */
var filterByStringPattern = exports.filterByStringPattern = (0, _fUtility.curry)(function (filterPattern, commits) {
  /**
   @method matcher
   @private
   @param {Object} commit - a commit object
   @return {boolean} whether there's a match
   */
  var matcher = function matcher(commit) {
    return (0, _fUtility.pipe)((0, _fUtility.split)("#"), (0, _fUtility.map)((0, _fUtility.split)(":")), (0, _fUtility.every)(stringMatcher(commit)))(filterPattern);
  };
  return (0, _fUtility.filter)(matcher, commits);
});