"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.createADefaultConfigFile = exports.ifNoConfig = exports.writeConfigFile = exports.processAndPrintWithConfig = exports.warnWriteOrLog = exports.handleKnownBadThings = exports.processData = exports.reader = exports.writeFile = exports.remapConfigData = exports.processGitCommits = exports.generateReport = exports.sideEffectCanonize = exports.sideEffect = exports.partyPrint = exports.partyData = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();
// import { trace } from "xtrace"


var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _gitlog = require("gitlog");

var _gitlog2 = _interopRequireDefault(_gitlog);

var _readData = require("read-data");

var _readData2 = _interopRequireDefault(_readData);

var _fUtility = require("f-utility");

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _fluture = require("fluture");

var _constants = require("./constants");

var _filters = require("./filters");

var _alias = require("./alias");

var _print = require("./print");

var _utils = require("./utils");

var _legend = require("./legend");

var _grouping = require("./grouping");

var _perCommit = require("./per-commit");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var NEWLINE = _constants.CHARACTER_LITERALS.NEWLINE;


var gotLog = (0, _fluture.encase)(_gitlog2.default);

/**
@method perCommit
@param {Object} lookup - the legend
@param {Object} x - the commit object
@return {Object} an updated object
*/
var perCommit = (0, _fUtility.curry)(function (timezone, lookup, x) {
  return (0, _fUtility.pipe)((0, _perCommit.addTimestampPerCommit)(timezone), _perCommit.addAliasesPerCommit, (0, _utils.macroLens)(_perCommit.convertStatusAndFilesPerCommit, "changes"), (0, _perCommit.addAnalysisPerCommit)(lookup))(x);
});

/**
@method partyData
@param {Object} config = the configuration object
@param {Object} lookup = the legend object
@param {Array} data - a list of commits
@return {Array} an augmented list of commits
*/
var partyData = exports.partyData = (0, _fUtility.curry)(function (config, lookup, data) {
  var grab = (0, _utils.preferredProp)(lookup, config, false);
  var filterMergeCommits = grab(_constants.ARGV_CONFIG.alias.m);
  var timezone = grab(_constants.ARGV_CONFIG.alias.t) || "utc";
  var collapseAuthors = grab(_constants.ARGV_CONFIG.alias.a[0]);
  var f = grab(_constants.ARGV_CONFIG.alias.f);
  var hasStringFilter = f && (0, _fUtility.isString)(f) && f.length > 0;
  return (0, _fUtility.pipe)(_utils.sortByDate, filterMergeCommits ? (0, _fUtility.reject)(_filters.isAMergeCommit) : _fUtility.I, (0, _fUtility.map)(perCommit(timezone, lookup)), hasStringFilter ? (0, _filters.filterByStringPattern)(f) : _fUtility.I, collapseAuthors ? (0, _grouping.collapseSuccessiveSameAuthor)(lookup) : _fUtility.I, _grouping.insertBanners)(data);
});

/**
@method partyPrint
@param {Object} config - the configuration object
@param {Object} lookup - the legend object
@param {Array} input - the commits
@return {string} colorized and stringified commits
*/
var partyPrint = exports.partyPrint = (0, _fUtility.curry)(function (config, lookup, input) {
  return (0, _fUtility.pipe)((0, _fUtility.map)((0, _print.colorize)(config, lookup)), (0, _fUtility.join)(NEWLINE))(input);
});
/**
@method prependLegend
@param {Object} lookup - the legend object
@param {Array} x - the commits
@return {string} colorized and stringified commits
*/
var prependLegend = (0, _fUtility.curry)(function (lookup, x) {
  return (0, _legend.printLegend)(lookup) + NEWLINE + x;
});

/**
@method sideEffect
@param {Function} effect - function to call
@param {Object} y - alias structure
@return {any} anything
*/
var sideEffect = exports.sideEffect = (0, _fUtility.curry)(function (effect, y) {
  return (0, _fUtility.pipe)(_fUtility.entries, (0, _fUtility.map)(function (_ref) {
    var _ref2 = _slicedToArray(_ref, 2),
        k = _ref2[0],
        v = _ref2[1];

    return Array.isArray(v) ? v.map(function (x) {
      return effect(k, x);
    }) : effect(k, v);
  }))(y);
});

/**
 @method sideEffectCanonize
 @param {Object} x - alias structure
 @return {null} nothing
 */
/* istanbul ignore next */
var sideEffectCanonize = exports.sideEffectCanonize = sideEffect(_alias.canonize);

var generateReport = exports.generateReport = (0, _fUtility.curry)(function (config, lookup, data) {
  var _lookup$aliases = lookup.aliases,
      aliases = _lookup$aliases === undefined ? {} : _lookup$aliases;
  // TODO: replace this with something that is a more explicitly managed side-effect
  /* istanbul ignore next */

  sideEffectCanonize(aliases);
  var filteredLookup = (0, _fUtility.filter)(function (z) {
    return z && z.matches;
  }, lookup);
  return (0, _fUtility.pipe)(partyData(config, filteredLookup), config.j ? _utils.j2 : (0, _fUtility.pipe)(partyPrint(config, filteredLookup), prependLegend(filteredLookup)))(data);
});

var processGitCommits = exports.processGitCommits = (0, _fUtility.curry)(function (config, lookup) {
  return (0, _fUtility.pipe)(gotLog, (0, _fUtility.map)(generateReport(config, lookup)))(config);
});

// || v.filter)
// filter: v.filter ? v.filter : null
var remapConfigData = exports.remapConfigData = (0, _fUtility.map)(function (v) {
  return (0, _fUtility.isObject)(v) && v.matches ? (0, _fUtility.merge)(v, {
    fn: _chalk2.default[v.color],
    matches: v.matches.map(_utils.stripDoubleBackslash)
  }) : v;
});
/* istanbul ignore next */
var writeFile = exports.writeFile = (0, _utils.binaryCallback)(
/* istanbul ignore next */
_fs2.default.writeFile.bind(_fs2.default),
/* istanbul ignore next */
function (e) {
  return (0, _utils.log)(e || "Wrote to file");
});
var reader = exports.reader = (0, _utils.unaryCallbackToFuture)(_readData2.default.yaml);

var processData = exports.processData = (0, _fUtility.curry)(function (config, F) {
  return (0, _fUtility.chain)((0, _fUtility.pipe)(remapConfigData, processGitCommits((0, _fUtility.merge)(_constants.DEFAULT_CONFIG, config))))(F);
});

/**
@method handleKnownBadThings
@param {Error} e - an error
@return {undefined}
*/
var handleKnownBadThings = exports.handleKnownBadThings = function handleKnownBadThings(e) {
  // TODO: really this function ought to be replaced Left
  if (e) {
    /* istanbul ignore next */
    if (e.code && e.path && e.code === "ENOENT") {
      if (e.path.substr(e.path.lastIndexOf("/")) === "/" + _constants.PARTYFILE) {
        var x = new Error(_constants.MAKE_A_GITPARTYRC_FILE);
        /* istanbul ignore next */
        (0, _utils.warn)(x);
        return x;
        /* istanbul ignore next */
      } else if (e.path === ".git/index") {
        var y = new Error(_constants.IT_ONLY_WORKS_IN_GIT_REPOS);
        /* istanbul ignore next */
        (0, _utils.warn)(y);
        return y;
      }
    }
    /* istanbul ignore next */
    (0, _utils.warn)(e);
  }
};

/* istanbul ignore next */
var warnWriteOrLog = exports.warnWriteOrLog = (0, _fUtility.curry)(
/* istanbul ignore next */
function (_ref3, F) {
  var o = _ref3.o;
  return (
    /* istanbul ignore next */
    (0, _fUtility.fork)(handleKnownBadThings, o ? writeFile(o) : _utils.log)(F)
  );
});

/* istanbul ignore next */
/**
@method processAndPrintWithConfig
@param {Object} config - a configuration object
@param {Object} input - hashes
@return {Future} future result
*/
var processAndPrintWithConfig = exports.processAndPrintWithConfig = (0, _fUtility.curry)(
/* istanbul ignore next */
function (config, input) {
  return (
    /* istanbul ignore next */
    (0, _fUtility.pipe)(
    // TODO: really what we want is a Future(Either) here, probably
    function (i) {
      return (0, _utils.fileOrError)(".git/index").and(reader(i));
    }, processData(config), warnWriteOrLog(config))(input)
  );
});

/**
@method writeConfigFile
@returns {Future} config file
*/
var writeConfigFile = exports.writeConfigFile = function writeConfigFile() {
  return (0, _fluture.encaseN2)(_fs2.default.writeFile)(_constants.PARTYFILE, _constants.BASE_CONFIG_FILE, "utf8");
};

/**
@method ifNoConfig
@param {function} x - a future returning function
@return {Future} future value
*/
var ifNoConfig = exports.ifNoConfig = function ifNoConfig(x) {
  return function () {
    return (0, _utils.fileExists)(_constants.PARTYFILE).or(x());
  };
};

/**
@method createADefaultConfigFile
@return {undefined} undefined
*/
var createADefaultConfigFile = exports.createADefaultConfigFile = ifNoConfig(writeConfigFile);