"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.collapseSuccessiveSameAuthor = exports.booleanMerge = exports.insertBanners = exports.createBannersFromGroups = undefined;

var _fUtility = require("f-utility");

var _arrayUnion = require("array-union");

var _arrayUnion2 = _interopRequireDefault(_arrayUnion);

var _mergeOptions = require("merge-options");

var _mergeOptions2 = _interopRequireDefault(_mergeOptions);

var _utils = require("./utils");

var _perCommit = require("./per-commit");

var _alias = require("./alias");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
@method createBannersFromGroups
@param {Object} grouped - a grouped object of commits
@return {Array} ungrouped array with banner objects inserted
*/
var createBannersFromGroups = exports.createBannersFromGroups = function createBannersFromGroups(grouped) {
  return (0, _fUtility.pipe)(_fUtility.keys, _utils.sortByDateObject, (0, _fUtility.reduce)(function (list, key) {
    return list.concat({ date: key, type: "banner" }, (0, _utils.sortByAuthorDate)(grouped[key]));
  }, []))(grouped);
};

/**
@method insertBanners
@param {Array} commits - a list of commit
@return {Array} commits with banners inserted (type: `banner`)
*/
var insertBanners = exports.insertBanners = (0, _fUtility.pipe)((0, _utils.groupBy)("date"), createBannersFromGroups);

/**
@method quash
@param {Array} x - an array of inputs
@returns {Array} a unique, filtered array with no null values and no pass-by-referents
*/
var quash = (0, _fUtility.pipe)(_utils.neue, (0, _fUtility.filter)(_fUtility.I), _arrayUnion2.default);
/**
@method booleanMerge
@param {Object} a - an object with boolean keys
@param {Object} b - another object with boolean keys
@returns {Object} an object which is the truthy merge of all keys
*/
var booleanMerge = exports.booleanMerge = (0, _fUtility.curry)(function (x, y) {
  return (0, _fUtility.pipe)(_fUtility.keys, (0, _fUtility.reduce)(function (out, key) {
    return (0, _fUtility.merge)(out, _defineProperty({}, key, x[key] || y[key]));
  }, {}))(x);
});

/**
@method relearn
@param {Object} commit - an object with a changes property
@return {Object} an updated object
*/
var relearn = (0, _fUtility.curry)(function (lookup, x) {
  return (0, _fUtility.pipe)((0, _utils.macroLens)(_perCommit.convertStatusAndFilesPerCommit, "changes"), (0, _perCommit.addAnalysisPerCommit)(lookup))(x);
});
/**
@method uniqueFiles
@param {*} _ - anything
@param {Array} files - a list of files
@return {Array} a unique list of files
*/
var uniqueFiles = function uniqueFiles(_, files) {
  return (0, _arrayUnion2.default)(files);
};

/**
@method collapseSuccessiveSameAuthor
@param {Object} lookup - the lookup / legend object
@param {Object} data - the list of commits
@return {Object} a potentially collapsed list of commits
*/
var collapseSuccessiveSameAuthor = exports.collapseSuccessiveSameAuthor = (0, _fUtility.curry)(function (lookup, data) {
  return (0, _fUtility.pipe)(_utils.sortByDate, (0, _fUtility.reduce)(function (list, next) {
    var copy = (0, _utils.neue)(list);
    var last = copy[copy.length - 1];
    if (next && last) {
      var authorsMatch = (0, _alias.getCanon)(last.authorName) === (0, _alias.getCanon)(next.authorName);
      var datesMatch = last.date === next.date;
      if (authorsMatch && datesMatch) {
        var raw = (0, _mergeOptions2.default)(last, next);
        var status = [].concat(last.status, next.status);
        var files = [].concat(last.files, next.files);
        var subject = last.subject + " + " + next.subject;
        var augmented = (0, _mergeOptions2.default)(raw, {
          hash: next.hash,
          files: files,
          subject: subject,
          status: status,
          multiple: true,
          hashes: quash([].concat(raw.hashes || [], last.hash, next.hash))
        });
        copy[copy.length - 1] = augmented; // eslint-disable-line fp/no-mutation
        return (0, _utils.sortByDate)(copy);
      }
    }
    return (0, _utils.sortByDate)(copy.concat(next));
  }, []), (0, _fUtility.map)((0, _fUtility.pipe)(relearn(lookup), (0, _utils.macroLens)(uniqueFiles, "files"))))(data);
});