#!/usr/bin/env node
"use strict";

var _path = require("path");

var _path2 = _interopRequireDefault(_path);

var _yargsParser = require("yargs-parser");

var _yargsParser2 = _interopRequireDefault(_yargsParser);

var _constants = require("./constants");

var _gitparty = require("./gitparty");

var _utils = require("./utils");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var flags = (0, _utils.flagify)(_constants.ARGV_CONFIG.alias);

var config = (0, _yargsParser2.default)(process.argv.slice(2), _constants.ARGV_CONFIG);
if (config.i) {
  (0, _gitparty.createADefaultConfigFile)().fork(
  // eslint-disable-next-line no-console
  function () {
    return console.warn("Unable to create a .gitpartyrc file");
  },
  // eslint-disable-next-line no-console
  function () {
    return console.log("Created a .gitpartyrc file");
  });
} else if (config.h) {
  /* eslint-disable no-console */
  /* eslint-disable max-len */
  console.log("\n# gitparty! it's a party for your git log\n\n" + flags.h + "\n  it's this thing you're reading now!\n\n## configuration\n\n" + flags.o + "\n  write the results to a file. usually useful in concert with the --json flag\n" + flags.j + "\n  return JSON output instead of converting to colorized strings\n" + flags.r + "\n  choose a different git repository\n" + flags.n + "\n  choose a number of total commits (default: 100)\n\n## filtering\n\n" + flags.a + "\n  merge commits if the authors are the same and the commit dates are the same\n" + flags.m + "\n  merge commits beginning with the string 'Merge '\n" + flags.f + "\n  filter commits based on a simple 'key:value' / 'key:value#key2:value2' base syntax:\n    * -f \"hash:80ca7f7\" / -f \"date:20-05-2018\"\n      lookup by exact string matching (default)\n    * -f \"subject:fix~\"\n      lookup by looser indexOf matching when there is a tilde \"~\" character at the end of the value\n    * -f \"subject:fix~#date:20-05-2018\"\n      lookup with multiple facets, separated by a hash \"#\" symbol\n    * -f \"author:brekk\"\n      when filtering by author, the alias lookup (if aliases have been defined) is used\n    * -f \"files:**/src/*.spec.js\"\n      when there are asterisks present in the value side (after the \":\")\n      and the key is an array, glob-style matching is performed\n    * -f \"analysis.config:true\"\n      when there is a period \".\" in the key, nested-key lookups will be performed\n    * -f \"x:true\" / -f \"x:false\"\n      when the value is either the literal string \"true\" or \"false\", it will be coerced\n      into a boolean\n\n## formatting\n\n" + flags.l + "\n  set a max width for author length (default: " + _constants.AUTHOR_LENGTH + ")\n" + flags.s + "\n  set a max width for subject length (default: " + _constants.SUBJECT_LENGTH + ")\n" + flags.b + "\n  set a max width for banner length (default: " + _constants.BANNER_LENGTH + ")\n" + flags.i + "\n  set a max width for banner indent (default: " + _constants.BANNER_INDENT + ")\n\n## aliases\n\nIf author aliases are present in the " + _constants.PARTYFILE + ", they will be used to identify authors in any gitparty functionality.\n\ne.g. in the " + _constants.PARTYFILE + ":\n\naliases:\n  brekk \uD83D\uDD25 :\n  - brekk\n  - Brekk\n\nWill result in the key (with the emoji) in place of any matched authors.\n");
  /* eslint-enable max-len */
} else {
  var input = _path2.default.resolve(process.cwd(), config.config || "./.gitpartyrc");
  (0, _gitparty.processAndPrintWithConfig)(config, input);
}
/* eslint-enable no-console */