"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.addTimestampPerCommit = exports.convertStatusAndFilesPerCommit = exports.addAliasesPerCommit = exports.addAnalysisPerCommit = exports.generateAnalysis = exports.filetypes = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fUtility = require("f-utility");

var _arrayUnion = require("array-union");

var _arrayUnion2 = _interopRequireDefault(_arrayUnion);

var _momentTimezone = require("moment-timezone");

var _momentTimezone2 = _interopRequireDefault(_momentTimezone);

var _utils = require("./utils");

var _filters = require("./filters");

var _alias = require("./alias");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

/**
@method grabAfterLastDot
@param {Array} strings - an array of strings
@return {Array} a list of potentially truncated strings
*/
var grabAfterLastDot = (0, _fUtility.map)(function (str) {
  return str.substr(str.lastIndexOf(".") + 1);
});

/**
@method filetypes
@param {Object} changes - a changes object, generated during analysis
@return {Array} an array of filetypes
*/
var filetypes = exports.filetypes = function filetypes(changes) {
  return (0, _fUtility.pipe)(_fUtility.keys, (0, _fUtility.reduce)(function (list, key) {
    return list.concat(grabAfterLastDot(changes[key]));
  }, []), _arrayUnion2.default,
  // eslint-disable-next-line fp/no-mutating-methods
  function (x) {
    return x.sort();
  })(changes);
};

/**
@method generateAnalysis
@param {Object} lookup - the legend object
@param {Object} commit - an object with changes
@param {Object} commit.changes - a changes object
@returns {Object} a commit object with an analysis property
*/
var generateAnalysis = exports.generateAnalysis = (0, _fUtility.curry)(function (lookup, _ref) {
  var changes = _ref.changes;
  return (0, _fUtility.pipe)(_fUtility.entries, (0, _fUtility.map)(function (_ref2) {
    var _ref3 = _slicedToArray(_ref2, 2),
        k = _ref3[0],
        matches = _ref3[1].matches;

    return [k, (0, _filters.anyFilesMatchFromObject)(changes, matches)];
  }), _fUtility.fromPairs)(lookup);
});

/**
@method addAnalysisPerCommit
@param {Object} lookup - the legend object
@param {Object} raw - the commit object
@return {Object} a standardized "commit" object
*/
var addAnalysisPerCommit = exports.addAnalysisPerCommit = (0, _fUtility.curry)(function (lookup, raw) {
  return (0, _fUtility.merge)(raw, {
    type: "commit",
    author: (0, _alias.getCanon)(raw.author),
    analysis: generateAnalysis(lookup, raw)
  });
});

/**
@method addAliasesPerCommit
@param {Object} commit - a commit object
@return {Object} a commit with some aliased properties
*/
var addAliasesPerCommit = exports.addAliasesPerCommit = (0, _fUtility.pipe)((0, _utils.aliasProperty)("authorName", "author"), (0, _utils.aliasProperty)("abbrevHash", "hash"));

/**
@method convertStatusAndFilesPerCommit
@param {Object} commit - a commit object
@return {Object} a "changes" object
*/
var convertStatusAndFilesPerCommit = exports.convertStatusAndFilesPerCommit = function convertStatusAndFilesPerCommit(commit) {
  /* eslint-disable require-jsdoc */
  var files = commit.files;
  // TODO: rewrite this so we don't have to rely on the i to count

  var arrayify = function arrayify(x) {
    return function (file, i) {
      var status = x.status[i];
      return [status, file];
    };
  };
  var flattenArrays = function flattenArrays(a, _ref4) {
    var _ref5 = _slicedToArray(_ref4, 2),
        k = _ref5[0],
        v = _ref5[1];

    return (0, _fUtility.merge)(a, _defineProperty({}, k, (a[k] || []).concat(v)));
  };
  return files.map(arrayify(commit)).reduce(flattenArrays, {});
  /* eslint-enable require-jsdoc */
};

/**
@method addTimestampPerCommit
@param {Object} commit - a commit object
@return {Object} a commit object with a timestamp property
*/
var addTimestampPerCommit = exports.addTimestampPerCommit = (0, _fUtility.curry)(function (timezone, commit) {
  var authorDate = commit.authorDate;

  var rel = (0, _momentTimezone2.default)(authorDate, "YYYY-MM-DD HH:mm:ss Z");
  var ms = rel.valueOf();
  var date = rel.tz(timezone).format("DD-MM-YYYY");
  return (0, _fUtility.merge)(commit, { ms: ms, date: date });
});