"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.colorize = exports.configureAndPrintCommit = exports.formatChanges = exports.formatSubject = exports.formatAuthor = exports.formatHash = exports.printAuthor = exports.configureAndPrintBanner = exports.drawTokens = exports.drawToken = undefined;

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _fUtility = require("f-utility");

var _pad = require("pad");

var _pad2 = _interopRequireDefault(_pad);

var _constants = require("./constants");

var _perCommit = require("./per-commit");

var _utils = require("./utils");

var _alias = require("./alias");

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// import { sideEffect, trace } from "xtrace"
var BLANK = _constants.CHARACTER_LITERALS.BLANK;


var padCharsStart = (0, _fUtility.curry)(function (length, str) {
  return (0, _pad2.default)(length, str, BLANK);
});
var padCharsEnd = (0, _fUtility.curry)(function (length, str) {
  return (0, _pad2.default)(str, length, BLANK);
});

var drawToken = exports.drawToken = (0, _fUtility.curry)(function (lookup, analysis, name) {
  // cold medina
  var _lookup$name = lookup[name],
      fn = _lookup$name.fn,
      key = _lookup$name.key;

  return analysis[name] ? _chalk2.default.black(fn(" " + key + " ")) : "   ";
});

var drawTokens = exports.drawTokens = (0, _fUtility.curry)(function (lookup, analysis) {
  return (0, _fUtility.pipe)(_fUtility.keys, (0, _fUtility.map)(drawToken(lookup, analysis)), (0, _fUtility.join)(""))(lookup);
});

var configureAndPrintBanner = exports.configureAndPrintBanner = (0, _fUtility.curry)(function (lookup, config, _ref) {
  var date = _ref.date;

  var grab = (0, _utils.preferredProp)(config, lookup);
  var bannerIndent = grab(_constants.BANNER_INDENT, "bannerIndent");
  var bannerLength = grab(_constants.BANNER_LENGTH, "bannerLength");
  return _chalk2.default.inverse(padCharsEnd(bannerLength, padCharsStart(bannerIndent, date)));
});

var printAuthor = exports.printAuthor = (0, _fUtility.curry)(function (author, length) {
  return (0, _fUtility.pipe)(_alias.getCanon, padCharsEnd(length), _chalk2.default.red)(author);
});
var printTypes = (0, _fUtility.pipe)(_perCommit.filetypes, (0, _fUtility.join)(BLANK));

var prepend = (0, _fUtility.curry)(function (str, str2) {
  return [str, str2].join(BLANK);
});

var formatHash = exports.formatHash = (0, _fUtility.pipe)(prepend("="), _chalk2.default.yellow);
var formatAuthor = exports.formatAuthor = (0, _fUtility.curry)(function (grab, author) {
  return (0, _fUtility.pipe)(grab(_constants.AUTHOR_LENGTH), printAuthor(author), prepend("$"))("authorLength");
});

var formatSubject = exports.formatSubject = (0, _fUtility.curry)(function (grab, subject) {
  return (0, _fUtility.pipe)(grab(_constants.SUBJECT_LENGTH), (0, _utils.summarize)(subject), prepend("-"))("subjectLength");
});

var formatChanges = exports.formatChanges = (0, _fUtility.pipe)(printTypes, prepend("|"));

var configureAndPrintCommit = exports.configureAndPrintCommit = (0, _fUtility.curry)(function (lookup, config, o) {
  var grab = (0, _utils.preferredProp)(config, lookup);
  return (0, _fUtility.pipe)((0, _utils.lens)(drawTokens(lookup), "analysis"), (0, _utils.lens)(formatHash, "hash"), (0, _utils.lens)(formatAuthor(grab), "author"), (0, _utils.lens)(formatSubject(grab), "subject"), (0, _utils.lens)(formatChanges, "changes"), function (_ref2) {
    var analysis = _ref2.analysis,
        hash = _ref2.hash,
        subject = _ref2.subject,
        author = _ref2.author,
        changes = _ref2.changes;
    return [analysis, hash, subject, author, changes].join(BLANK);
  })(o);
});

var colorize = exports.colorize = (0, _fUtility.curry)(function (config, lookup, raw) {
  return (raw.type === "banner" ? configureAndPrintBanner : configureAndPrintCommit)(lookup, config, raw);
});