"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.flagify = exports.groupBy = exports.fileExists = exports.fileOrError = exports.fileAccess = exports.indexAny = exports.processKeysByLookup = exports.stripDoubleBackslash = exports.preferredProp = exports.binaryCallback = exports.sortByAuthorDate = exports.sortByDateObject = exports.sortByDateKey = exports.sortByDate = exports.sortByKeyWithWrapper = exports.lens = exports.macroLens = exports.j2 = exports.aliasProperty = exports.summarize = exports.neue = exports.box = exports.warn = exports.log = exports.unaryCallbackToFuture = undefined;

var _slicedToArray = function () { function sliceIterator(arr, i) { var _arr = []; var _n = true; var _d = false; var _e = undefined; try { for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) { _arr.push(_s.value); if (i && _arr.length === i) break; } } catch (err) { _d = true; _e = err; } finally { try { if (!_n && _i["return"]) _i["return"](); } finally { if (_d) throw _e; } } return _arr; } return function (arr, i) { if (Array.isArray(arr)) { return arr; } else if (Symbol.iterator in Object(arr)) { return sliceIterator(arr, i); } else { throw new TypeError("Invalid attempt to destructure non-iterable instance"); } }; }();

var _fs = require("fs");

var _fs2 = _interopRequireDefault(_fs);

var _groupArrayBy = require("group-array-by");

var _groupArrayBy2 = _interopRequireDefault(_groupArrayBy);

var _chalk = require("chalk");

var _chalk2 = _interopRequireDefault(_chalk);

var _fUtility = require("f-utility");

var _fluture = require("fluture");

var _fluture2 = _interopRequireDefault(_fluture);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }
// import { trace } from "xtrace"


var unaryCallbackToFuture = exports.unaryCallbackToFuture = (0, _fUtility.curry)(function (fn, x) {
  return new _fluture2.default(function (rej, res) {
    return fn(x, function (e, d) {
      return e ? rej(e) : res(d);
    });
  });
});

var log = exports.log = console.log; // eslint-disable-line no-console
var warn = exports.warn = console.warn; // eslint-disable-line no-console
/**
@method box
@param {*} x - anything
@return {Array} something in an array
*/
var box = exports.box = function box(x) {
  return Array.isArray(x) ? x : [x];
};
/**
@method neue
@param {Object|Array} x - something that might be an array or an object
@return {Object|Array} a cloned copy of said array or object
*/
var neue = exports.neue = function neue(x) {
  return x && Array.isArray(x) ? [].concat(x) : (0, _fUtility.merge)({}, x);
};

/**
@method summarize
@param {number} limit - the max length of the string, + 3
@return {string} a summarized string, within the goal limits + 3
*/
var summarize = exports.summarize = (0, _fUtility.curry)(function (str, limit) {
  return (0, _fUtility.padEnd)(limit + 3, " ", str.substr(0, limit) + ("" + (str.length > limit ? "..." : "")));
});

/**
@method aliasProperty
@param {string} property - a string property key
@param {string} propAlias - a string property key alias
@param {Object} x - some object
@return {Object} object with aliased property, or no change
*/
var aliasProperty = exports.aliasProperty = (0, _fUtility.curry)(function (property, propAlias, x) {
  return typeof x[property] !== "undefined" ? (0, _fUtility.merge)(x, _defineProperty({}, propAlias, x[property])) : neue(x);
});

/**
@method j2
@param {*} x - anything
@return {string} json stringified stuff with a 2 space indent
*/
var j2 = exports.j2 = function j2(x) {
  return JSON.stringify(x, null, 2);
};

/**
@method macroLens
@param {Function} fn - a macroLens function (copy, property) => {}
@param {string} prop - a property to macroLens
@param {Object} target - an object which has a property to macroLens
@return {Object} a cloned object with a modified property
*/
var macroLens = exports.macroLens = (0, _fUtility.curry)(function (fn, property, target) {
  var copy = neue(target);
  return copy && property ? (0, _fUtility.merge)(copy, _defineProperty({}, property, fn(copy, copy[property]))) : copy;
});

var lens = exports.lens = (0, _fUtility.curry)(function (fn, property, target) {
  var copy = neue(target);
  return copy && property ? neue((0, _fUtility.merge)(copy, _defineProperty({}, property, fn(copy[property])))) : copy;
});

/**
@method sortByKeyWithWrapper
@param {boolean} ascendingSort - sort ascending?
@param {Function} wrap - a function to wrap the comparitors with
@param {string} key - a key which is shared across both comparitors
@param {Array} arr - an array to be sorted
@return {Array} a sorted array
*/
var sortByKeyWithWrapper = exports.sortByKeyWithWrapper = (0, _fUtility.curry)(function (ascendingSort, wrap, key, arr) {
  return (
    // eslint-disable-next-line fp/no-mutating-methods
    neue(arr).sort(function (_ref, _ref2) {
      var a = _ref[key];
      var b = _ref2[key];
      return ascendingSort ? wrap(b) - wrap(a) : wrap(a) - wrap(b);
    })
  );
});

// eslint-disable-next-line fp/no-mutating-methods
var sortByDate = exports.sortByDate = sortByKeyWithWrapper(true, _fUtility.I, "ms");
// eslint-disable-next-line require-jsdoc
var tomorrow = function tomorrow(x) {
  return new Date(x);
};
var sortByDateKey = exports.sortByDateKey = sortByKeyWithWrapper(true, tomorrow);
// eslint-disable-next-line require-jsdoc
var prettyPrintDate = function prettyPrintDate(x) {
  return tomorrow(x.split("-").reverse() // eslint-disable-line fp/no-mutating-methods
  );
};

// eslint-disable-next-line require-jsdoc
var sortByDateObject = exports.sortByDateObject = function sortByDateObject(k) {
  return (
    // eslint-disable-next-line fp/no-mutating-methods
    k.sort(function (a, b) {
      return prettyPrintDate(b) - prettyPrintDate(a);
    })
  );
};
var sortByAuthorDate = exports.sortByAuthorDate = sortByDateKey("authorDate");

var binaryCallback = exports.binaryCallback = (0, _fUtility.curry)(function (orig, cb, output, data) {
  orig(output, data, cb);
});
var preferredProp = exports.preferredProp = (0, _fUtility.curry)(function (a, b, def, key) {
  var _a = (0, _fUtility.prop)(key, neue(a));
  var _b = (0, _fUtility.prop)(key, neue(b));
  return _a || _b || def;
});
// eslint-disable-next-line require-jsdoc
var stripDoubleBackslash = exports.stripDoubleBackslash = function stripDoubleBackslash(w) {
  return w.replace(/^\\/g, "");
};

var processKeysByLookup = exports.processKeysByLookup = (0, _fUtility.curry)(function (fnLookup, o) {
  return (0, _fUtility.pipe)(_fUtility.entries, (0, _fUtility.map)(function (_ref3) {
    var _ref4 = _slicedToArray(_ref3, 2),
        k = _ref4[0],
        v = _ref4[1];

    return function (_ref5) {
      var _ref6 = _slicedToArray(_ref5, 2),
          k1 = _ref6[0],
          v1 = _ref6[1];

      return k1 === k ? [k, v(v1)] : null;
    };
  }), function (table) {
    return (0, _fUtility.pipe)(_fUtility.entries, (0, _fUtility.ap)(table), (0, _fUtility.filter)(_fUtility.I))(o);
  })(fnLookup);
});

var indexAny = exports.indexAny = (0, _fUtility.curry)(function (look, str) {
  return str.indexOf(look) > -1;
});

/**
@method fileAccess
@param {boolean} returnFalse - return false?
@param {string} file - a file reference
@returns {Future} some future
*/
var fileAccess = exports.fileAccess = (0, _fUtility.curry)(function (returnFalse, file) {
  return new _fluture2.default(function (rej, res) {
    return _fs2.default.access(file,
    /* ------------------ */
    _fs2.default.constants.F_OK, function (e) {
      return e ? rej(returnFalse ? false : e) : res(true);
    });
  });
});

/**
@method fileOrError
@param {string} file - a file reference
@returns {Future} true or an error
*/
var fileOrError = exports.fileOrError = fileAccess(false);
/**
@method fileExists
@param {string} file - a file reference
@returns {Future} eventual boolean value
*/
var fileExists = exports.fileExists = fileAccess(true);

var groupBy = exports.groupBy = (0, _fUtility.curry)(function (fn, iter) {
  return (0, _groupArrayBy2.default)((0, _fUtility.isFunction)(fn) ? fn : (0, _fUtility.prop)(fn), iter);
});

var flagify = exports.flagify = (0, _fUtility.pipe)(_fUtility.toPairs, (0, _fUtility.map)((0, _fUtility.pipe)(function (_ref7) {
  var _ref8 = _slicedToArray(_ref7, 2),
      x = _ref8[0],
      y = _ref8[1];

  return box(x).concat(y);
}, function (x) {
  return x.sort(function (a, b) {
    return a.length - b.length;
  });
}, // eslint-disable-line fp/no-mutating-methods
function (x) {
  return [x[0], x];
}, function (_ref9) {
  var _ref10 = _slicedToArray(_ref9, 2),
      key = _ref10[0],
      flags = _ref10[1];

  return [key, (0, _fUtility.map)(function (x) {
    return x.length === 1 ? "-" + _chalk2.default.yellow(x) : "--" + _chalk2.default.yellow(x);
  }, flags)];
}, function (_ref11) {
  var _ref12 = _slicedToArray(_ref11, 2),
      k = _ref12[0],
      v = _ref12[1];

  return _defineProperty({}, k, (0, _fUtility.join)(" / ", v));
})), (0, _fUtility.reduce)(_fUtility.merge, {}));